import { chemicalSchema } from '@/models/chemicalSchema';
import { drugSchema } from '@/models/drugSchema';
import { targetSearchNameResult } from '@/models/targetSearchNameResult';
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { Chemical, Drug, TargetSearchNameResult } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { apiPath } from '../apiPath';

const QUERY_COLUMN_NAME = 'name';

const getDrugsNamesForTarget = async (target: string): Promise<TargetSearchNameResult[]> => {
  const response = await axiosInstanceNewAPI.get<TargetSearchNameResult[]>(
    apiPath.getDrugsStringWithColumnsTarget(QUERY_COLUMN_NAME, target),
  );

  const isDataValid = (obj: unknown): obj is TargetSearchNameResult =>
    validateDataUsingZodSchema(obj, targetSearchNameResult);

  return response.data.filter(isDataValid);
};

const getDrugsByName = async (drugName: string): Promise<Drug[]> => {
  const response = await axiosInstanceNewAPI.get<Drug[]>(apiPath.getDrugsStringWithQuery(drugName));

  const isDataValid = (obj: unknown): obj is Drug => validateDataUsingZodSchema(obj, drugSchema);

  return response.data.filter(isDataValid);
};

export const getDrugsForBioEntityDrawerTarget = createAsyncThunk(
  'drawer/getDrugsForBioEntityDrawerTarget',
  async (target: string): Promise<Drug[]> => {
    const drugsNames = await getDrugsNamesForTarget(target);
    const drugsArrays = await Promise.all(
      drugsNames.map(({ name }) => getDrugsByName(encodeURIComponent(name))),
    );
    const drugs = drugsArrays.flat();

    return drugs;
  },
);

const getChemicalsNamesForTarget = async (target: string): Promise<TargetSearchNameResult[]> => {
  const response = await axiosInstanceNewAPI.get<TargetSearchNameResult[]>(
    apiPath.getChemicalsStringWithColumnsTarget(QUERY_COLUMN_NAME, target),
  );

  const isDataValid = (obj: unknown): obj is TargetSearchNameResult =>
    validateDataUsingZodSchema(obj, targetSearchNameResult);

  return response.data.filter(isDataValid);
};

const getChemicalsByName = async (chemicalName: string): Promise<Chemical[]> => {
  const response = await axiosInstanceNewAPI.get<Chemical[]>(
    apiPath.getChemicalsStringWithQuery(chemicalName),
  );

  const isDataValid = (obj: unknown): obj is Chemical =>
    validateDataUsingZodSchema(obj, chemicalSchema);

  return response.data.filter(isDataValid);
};

export const getChemicalsForBioEntityDrawerTarget = createAsyncThunk(
  'drawer/getChemicalsForBioEntityDrawerTarget',
  async (target: string): Promise<Chemical[]> => {
    const chemicalsNames = await getChemicalsNamesForTarget(target);
    const chemicalsArrays = await Promise.all(
      chemicalsNames.map(({ name }) => getChemicalsByName(encodeURIComponent(name))),
    );
    const chemicals = chemicalsArrays.flat();

    return chemicals;
  },
);
