import { createAsyncThunk } from '@reduxjs/toolkit';
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { OverlayBioEntity } from '@/types/models';
import { OverlayBioEntityRender } from '@/types/OLrendering';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import {
  getValidOverlayBioEntities,
  parseOverlayBioEntityToOlRenderingFormat,
} from './overlayBioEntity.utils';
import { apiPath } from '../apiPath';
import { modelsIdsSelector } from '../models/models.selectors';
import type { RootState } from '../store';
import { setMapBackground } from '../map/map.slice';
import { emptyBackgroundIdSelector } from '../backgrounds/background.selectors';
import { overlaySelector, userOverlaySelector } from '../overlays/overlays.selectors';

type GetOverlayBioEntityThunkProps = {
  overlayId: number;
  modelId: number;
};

export const getOverlayBioEntity = createAsyncThunk(
  'overlayBioEntity/getOverlayBioEntity',
  async ({
    overlayId,
    modelId,
  }: GetOverlayBioEntityThunkProps): Promise<OverlayBioEntityRender[] | undefined> => {
    const response = await axiosInstanceNewAPI.get<OverlayBioEntity[]>(
      apiPath.getOverlayBioEntity({ overlayId, modelId }),
      {
        withCredentials: true,
      },
    );

    const validOverlayBioEntities = getValidOverlayBioEntities(response.data);

    if (validOverlayBioEntities) {
      return parseOverlayBioEntityToOlRenderingFormat(validOverlayBioEntities, overlayId);
    }

    return undefined;
  },
);

type GetOverlayBioEntityForAllModelsThunkProps = { overlayId: number };

export const getOverlayBioEntityForAllModels = createAsyncThunk<
  void,
  GetOverlayBioEntityForAllModelsThunkProps,
  { state: RootState }
>(
  'overlayBioEntity/getOverlayBioEntityForAllModels',
  async ({ overlayId }, { dispatch, getState }): Promise<void> => {
    const state = getState();
    const modelsIds = modelsIdsSelector(state);

    const asyncGetOverlayBioEntityFunctions = modelsIds.map(id =>
      dispatch(getOverlayBioEntity({ overlayId, modelId: id })),
    );

    await Promise.all(asyncGetOverlayBioEntityFunctions);
  },
);

type GetInitOverlaysProps = { overlaysId: number[] };

export const getInitOverlays = createAsyncThunk<void, GetInitOverlaysProps, { state: RootState }>(
  'appInit/getInitOverlays',
  async ({ overlaysId }, { dispatch, getState }): Promise<void> => {
    const state = getState();

    const emptyBackgroundId = emptyBackgroundIdSelector(state);
    if (emptyBackgroundId) {
      dispatch(setMapBackground(emptyBackgroundId));
      PluginsEventBus.dispatchEvent('onBackgroundOverlayChange', emptyBackgroundId);
    }

    overlaysId.forEach(id => {
      const userOverlay = userOverlaySelector(state, id);
      const overlay = overlaySelector(state, id);
      const eventData = userOverlay || overlay;

      if (eventData) {
        PluginsEventBus.dispatchEvent('onShowOverlay', eventData);
      }

      dispatch(getOverlayBioEntityForAllModels({ overlayId: id }));
    });
  },
);
