/* eslint-disable no-magic-numbers */
import {
  createdOverlayFileSchema,
  createdOverlaySchema,
  mapOverlay,
  uploadedOverlayFileContentSchema,
} from '@/models/mapOverlay';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { CreatedOverlay, CreatedOverlayFile, MapOverlay } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import { apiPath } from '../apiPath';
import { CHUNK_SIZE } from './overlays.constants';
import { closeModal } from '../modal/modal.slice';
import type { RootState } from '../store';

export const getAllPublicOverlaysByProjectId = createAsyncThunk(
  'overlays/getAllPublicOverlaysByProjectId',
  async (projectId: string): Promise<MapOverlay[]> => {
    const response = await axiosInstance.get<MapOverlay[]>(
      apiPath.getAllOverlaysByProjectIdQuery(projectId, { publicOverlay: true }),
    );

    const isDataValid = validateDataUsingZodSchema(response.data, z.array(mapOverlay));

    return isDataValid ? response.data : [];
  },
);

export const getAllUserOverlaysByCreator = createAsyncThunk(
  'overlays/getAllUserOverlaysByCreator',
  async (_, { getState }): Promise<MapOverlay[]> => {
    const state = getState() as RootState;
    const creator = state.user.login;
    if (!creator) return [];

    const response = await axiosInstance<MapOverlay[]>(
      apiPath.getAllUserOverlaysByCreatorQuery({
        creator,
        publicOverlay: false,
      }),
      {
        withCredentials: true,
      },
    );

    const isDataValid = validateDataUsingZodSchema(response.data, z.array(mapOverlay));

    const sortByOrder = (userOverlayA: MapOverlay, userOverlayB: MapOverlay): number => {
      if (userOverlayA.order > userOverlayB.order) return 1;
      return -1;
    };

    const sortedUserOverlays = response.data.sort(sortByOrder);

    return isDataValid ? sortedUserOverlays : [];
  },
);

/** UTILS */

type CreateFileArgs = {
  filename: string;
  content: string;
};

const createFile = async ({ filename, content }: CreateFileArgs): Promise<CreatedOverlayFile> => {
  const fileParams = {
    filename: `C:\\fakepath\\${filename}`,
    length: content.length.toString(),
  };

  const response = await axiosInstance.post(
    apiPath.createOverlayFile(),
    new URLSearchParams(fileParams),
    {
      withCredentials: true,
    },
  );

  const isDataValid = validateDataUsingZodSchema(response.data, createdOverlayFileSchema);
  return isDataValid ? response.data : undefined;
};

type UploadContentArgs = {
  createdFile: CreatedOverlayFile;
  overlayContent: string;
};
const uploadContent = async ({ createdFile, overlayContent }: UploadContentArgs): Promise<void> => {
  const data = new Uint8Array(new TextEncoder().encode(overlayContent));
  let uploadedLength = 0;

  const sendChunk = async (): Promise<void> => {
    if (uploadedLength >= data.length) {
      return;
    }

    const chunk = data.slice(uploadedLength, uploadedLength + CHUNK_SIZE);

    const responeJSON = await fetch(
      `${process.env.NEXT_PUBLIC_BASE_API_URL}/${apiPath.uploadOverlayFileContent(createdFile.id)}`,
      {
        method: 'POST',
        credentials: 'include',
        body: chunk,
      },
    );

    const response = await responeJSON.json();
    validateDataUsingZodSchema(response, uploadedOverlayFileContentSchema);

    uploadedLength += chunk.length;
    sendChunk();
  };

  await sendChunk();
};

type CreatedOverlayArgs = {
  createdFile: CreatedOverlayFile;
  description: string;
  name: string;
  type: string;
  projectId: string;
};

const creteOverlay = async ({
  createdFile,
  description,
  type,
  name,
  projectId,
}: CreatedOverlayArgs): Promise<CreatedOverlay | undefined> => {
  const data = {
    name,
    description,
    filename: createdFile.filename,
    googleLicenseConsent: false.toString(),
    type,
    fileId: createdFile.id.toString(),
  };

  const overlay = new URLSearchParams(data);

  const response = await axiosInstance.post<CreatedOverlay>(
    apiPath.createOverlay(projectId),
    overlay,
    {
      withCredentials: true,
    },
  );

  PluginsEventBus.dispatchEvent('onAddDataOverlay', response.data);

  const isDataValid = validateDataUsingZodSchema(response.data, createdOverlaySchema);

  return isDataValid ? response.data : undefined;
};

type AddOverlayArgs = {
  filename: string;
  content: string;
  description: string;
  type: string;
  name: string;
  projectId: string;
};

export const addOverlay = createAsyncThunk(
  'overlays/addOverlay',
  async (
    { filename, content, description, name, type, projectId }: AddOverlayArgs,
    { dispatch },
  ): Promise<void> => {
    const createdFile = await createFile({
      filename,
      content,
    });

    await uploadContent({
      createdFile,
      overlayContent: content,
    });

    await creteOverlay({
      createdFile,
      description,
      name,
      type,
      projectId,
    });

    dispatch(getAllUserOverlaysByCreator());
  },
);

export const updateOverlays = createAsyncThunk(
  'overlays/updateOverlays',
  async (userOverlays: MapOverlay[]): Promise<void> => {
    const userOverlaysPromises = userOverlays.map(userOverlay =>
      axiosInstance.patch<MapOverlay>(
        apiPath.updateOverlay(userOverlay.idObject),
        {
          overlay: userOverlay,
        },
        {
          withCredentials: true,
        },
      ),
    );

    const userOverlaysResponses = await Promise.all(userOverlaysPromises);

    const updatedUserOverlays = userOverlaysResponses.map(
      updatedUserOverlay => updatedUserOverlay.data,
    );

    validateDataUsingZodSchema(updatedUserOverlays, z.array(mapOverlay));
  },
);

export const removeOverlay = createAsyncThunk(
  'overlays/removeOverlay',
  async ({ overlayId }: { overlayId: number }, thunkApi): Promise<void> => {
    await axiosInstance.delete(apiPath.removeOverlay(overlayId), {
      withCredentials: true,
    });

    PluginsEventBus.dispatchEvent('onRemoveDataOverlay', overlayId);
    await thunkApi.dispatch(getAllUserOverlaysByCreator());
    thunkApi.dispatch(closeModal());
  },
);
